use utf8;
use strict;
use warnings FATAL => 'all';

package Xyzzy::Request;

use Clarity -self;

use Scalar::Util qw(reftype);

use Xyzzy::Response;
use Xyzzy::Status;

field ctx;
field cfg => sub { shift->ctx->cfg };
field cgi => sub { shift->ctx->cgi };

our @EXPORT_BASE = qw(param);

sub secure {
	return lc($self->cgi->https) eq 'on';
}

sub msie {
	return $self->{msie} if exists $self->{msie};
	my $ua = $self->cgi->user_agent;
	return $self->{msie} = defined $ua && index($ua, 'MSIE') != -1;
}

sub error {
    return new Xyzzy::Response(status => new Xyzzy::Status(@_));
}

sub param(*@) {
	my $name = shift;
	return $name->cgi->param(@_)
		if ref $name;
	my $package = caller;
	my $init = shift;
	my $or = defined $init
		? ref $init && reftype $init eq 'CODE'
			? ' // $init->($self, $name, $package)'
			: ' // Carp::confess($init)'
		: '';
	my $code = <<EOS;
		package $package;
		sub $name {
			my \$self = shift;
			return \$self->{\$name} if exists \$self->{\$name};
			return \$self->{\$name} = \$self->cgi->param(\$name)$or;
		}
EOS
	my $err = do { local $@; eval $code; $@ };
	confess $err if $err;
}

sub DESTROY {} # don't try to autoload this

sub AUTOLOAD {
	my $sub = our $AUTOLOAD;
	my $off = rindex($sub, '::');
	confess "no package name in '$sub'"
		if $off == -1;
	my $pkg = substr($sub, 0, $off + 2, '');
	my $cgi = $self->cgi;
	unless($cgi->can($sub)) {
		substr($pkg, -2) = '';
		confess "Can't locate object method \"$sub\" via package \"$pkg\"";
	}
	my $code = "sub $sub { my \$self = shift; return \$self->cgi->$sub(\@_) }";
	my $err = do { local $@; eval $code; $@ };
	confess $err if $err;
	return $cgi->$sub(@_);
}
